@@ -7,10 +7,11 @@ estimator, as a chain of transforms and estimators.
 #         Virgile Fritsch
 #         Alexandre Gramfort
 #         Lars Buitinck
-# Licence: BSD
+# License: BSD
 
 from collections import defaultdict
 from warnings import warn
+from abc import ABCMeta, abstractmethod
 
 import numpy as np
 from scipy import sparse
@@ -20,12 +21,69 @@ from .externals.joblib import Parallel, delayed
 from .externals import six
 from .utils import tosequence
 from .utils.metaestimators import if_delegate_has_method
-from .externals.six import iteritems
 
 __all__ = ['Pipeline', 'FeatureUnion']
 
 
-class Pipeline(BaseEstimator):
+class _BasePipeline(six.with_metaclass(ABCMeta, BaseEstimator)):
+    """Handles parameter management for classifiers composed of named steps.
+    """
+
+    @abstractmethod
+    def __init__(self):
+        pass
+
+    def _replace_step(self, steps_attr, name, new_val):
+        # assumes `name` is a valid step name
+        new_steps = getattr(self, steps_attr)[:]
+        for i, (step_name, _) in enumerate(new_steps):
+            if step_name == name:
+                new_steps[i] = (name, new_val)
+                break
+        setattr(self, steps_attr, new_steps)
+
+    def _get_params(self, steps_attr, deep=True):
+        out = super(_BasePipeline, self).get_params(deep=False)
+        if not deep:
+            return out
+        steps = getattr(self, steps_attr)
+        out.update(steps)
+        for name, estimator in steps:
+            if estimator is None:
+                continue
+            for key, value in six.iteritems(estimator.get_params(deep=True)):
+                out['%s__%s' % (name, key)] = value
+        return out
+
+    def _set_params(self, steps_attr, **params):
+        # Ensure strict ordering of parameter setting:
+        # 1. All steps
+        if steps_attr in params:
+            setattr(self, steps_attr, params.pop(steps_attr))
+        # 2. Step replacement
+        step_names, _ = zip(*getattr(self, steps_attr))
+        for name in list(six.iterkeys(params)):
+            if '__' not in name and name in step_names:
+                self._replace_step(steps_attr, name, params.pop(name))
+        # 3. Step parameters and other initilisation arguments
+        super(_BasePipeline, self).set_params(**params)
+        return self
+
+    def _validate_names(self, names):
+        if len(set(names)) != len(names):
+            raise ValueError('Names provided are not unique: '
+                             '{0!r}'.format(list(names)))
+        invalid_names = set(names).intersection(self.get_params(deep=False))
+        if invalid_names:
+            raise ValueError('Step names conflict with constructor arguments: '
+                             '{0!r}'.format(sorted(invalid_names)))
+        invalid_names = [name for name in names if '__' in name]
+        if invalid_names:
+            raise ValueError('Step names must not contain __: got '
+                             '{0!r}'.format(invalid_names))
+
+
+class Pipeline(_BasePipeline):
     """Pipeline of transforms with a final estimator.
 
     Sequentially apply a list of transforms and a final estimator.
@@ -37,6 +95,9 @@ class Pipeline(BaseEstimator):
     cross-validated together while setting different parameters.
     For this, it enables setting parameters of the various steps using their
     names and the parameter name separated by a '__', as in the example below.
+    A step's estimator may be replaced entirely by setting the parameter
+    with its name to another estimator, or a transformer removed by setting
+    to None.
 
     Read more in the :ref:`User Guide <pipeline>`.
 
@@ -75,55 +136,79 @@ class Pipeline(BaseEstimator):
     Pipeline(steps=[...])
     >>> prediction = anova_svm.predict(X)
     >>> anova_svm.score(X, y)                        # doctest: +ELLIPSIS
-    0.77...
+    0.829...
     >>> # getting the selected features chosen by anova_filter
     >>> anova_svm.named_steps['anova'].get_support()
     ... # doctest: +NORMALIZE_WHITESPACE
-    array([ True,  True,  True, False, False,  True, False,  True,  True, True,
-           False, False,  True, False,  True, False, False, False, False,
-           True], dtype=bool)
+    array([False, False,  True,  True, False, False, True,  True, False,
+           True,  False,  True,  True, False, True,  False, True, True,
+           False, False], dtype=bool)
     """
 
     # BaseEstimator interface
 
     def __init__(self, steps):
-        names, estimators = zip(*steps)
-        if len(dict(steps)) != len(steps):
-            raise ValueError("Provided step names are not unique: %s" % (names,))
-
         # shallow copy of steps
         self.steps = tosequence(steps)
-        transforms = estimators[:-1]
+        self._validate_steps()
+
+    def get_params(self, deep=True):
+        """Get parameters for this estimator.
+
+        Parameters
+        ----------
+        deep: boolean, optional
+            If True, will return the parameters for this estimator and
+            contained subobjects that are estimators.
+
+        Returns
+        -------
+        params : mapping of string to any
+            Parameter names mapped to their values.
+        """
+        return self._get_params('steps', deep=deep)
+
+    def set_params(self, **kwargs):
+        """Set the parameters of this estimator.
+
+        Valid parameter keys can be listed with ``get_params()``.
+
+        Returns
+        -------
+        self
+        """
+        self._set_params('steps', **kwargs)
+        return self
+
+    def _validate_steps(self):
+        names, estimators = zip(*self.steps)
+
+        # validate names
+        self._validate_names(names)
+
+        # validate estimators
+        transformers = estimators[:-1]
         estimator = estimators[-1]
 
-        for t in transforms:
+        for t in transformers:
+            if t is None:
+                continue
             if (not (hasattr(t, "fit") or hasattr(t, "fit_transform")) or not
                     hasattr(t, "transform")):
-                raise TypeError("All intermediate steps of the chain should "
-                                "be transforms and implement fit and transform"
-                                " '%s' (type %s) doesn't)" % (t, type(t)))
+                raise TypeError("All intermediate steps should be "
+                                "transformers and implement fit and transform."
+                                " '%s' (type %s) doesn't" % (t, type(t)))
 
-        if not hasattr(estimator, "fit"):
-            raise TypeError("Last step of chain should implement fit "
-                            "'%s' (type %s) doesn't)"
+        # We allow last estimator to be None as an identity transformation
+        if estimator is not None and not hasattr(estimator, "fit"):
+            raise TypeError("Last step of Pipeline should implement fit. "
+                            "'%s' (type %s) doesn't"
                             % (estimator, type(estimator)))
 
     @property
     def _estimator_type(self):
         return self.steps[-1][1]._estimator_type
 
-    def get_params(self, deep=True):
-        if not deep:
-            return super(Pipeline, self).get_params(deep=False)
-        else:
-            out = self.named_steps
-            for name, step in six.iteritems(self.named_steps):
-                for key, value in six.iteritems(step.get_params(deep=True)):
-                    out['%s__%s' % (name, key)] = value
-
-            out.update(super(Pipeline, self).get_params(deep=False))
-            return out
-
     @property
     def named_steps(self):
         return dict(self.steps)
@@ -134,22 +219,30 @@ class Pipeline(BaseEstimator):
 
     # Estimator interface
 
-    def _pre_transform(self, X, y=None, **fit_params):
-        fit_params_steps = dict((step, {}) for step, _ in self.steps)
+    def _fit(self, X, y=None, **fit_params):
+        self._validate_steps()
+        fit_params_steps = dict((name, {}) for name, step in self.steps
+                                if step is not None)
         for pname, pval in six.iteritems(fit_params):
             step, param = pname.split('__', 1)
             fit_params_steps[step][param] = pval
         Xt = X
         for name, transform in self.steps[:-1]:
-            if hasattr(transform, "fit_transform"):
+            if transform is None:
+                pass
+            elif hasattr(transform, "fit_transform"):
                 Xt = transform.fit_transform(Xt, y, **fit_params_steps[name])
             else:
                 Xt = transform.fit(Xt, y, **fit_params_steps[name]) \
                               .transform(Xt)
+        if self._final_estimator is None:
+            return Xt, {}
         return Xt, fit_params_steps[self.steps[-1][0]]
 
     def fit(self, X, y=None, **fit_params):
-        """Fit all the transforms one after the other and transform the
+        """Fit the model
+
+        Fit all the transforms one after the other and transform the
         data, then fit the transformed data using the final estimator.
 
         Parameters
@@ -157,17 +250,31 @@ class Pipeline(BaseEstimator):
         X : iterable
             Training data. Must fulfill input requirements of first step of the
             pipeline.
+
         y : iterable, default=None
             Training targets. Must fulfill label requirements for all steps of
             the pipeline.
+
+        **fit_params : dict of string -> object
+            Parameters passed to the ``fit`` method of each step, where
+            each parameter name is prefixed such that parameter ``p`` for step
+            ``s`` has key ``s__p``.
+
+        Returns
+        -------
+        self : Pipeline
+            This estimator
         """
-        Xt, fit_params = self._pre_transform(X, y, **fit_params)
-        self.steps[-1][-1].fit(Xt, y, **fit_params)
+        Xt, fit_params = self._fit(X, y, **fit_params)
+        if self._final_estimator is not None:
+            self._final_estimator.fit(Xt, y, **fit_params)
         return self
 
     def fit_transform(self, X, y=None, **fit_params):
-        """Fit all the transforms one after the other and transform the
-        data, then use fit_transform on transformed data using the final
+        """Fit the model and transform with the final estimator
+
+        Fits all the transforms one after the other and transforms the
+        data, then uses fit_transform on transformed data with the final
         estimator.
 
         Parameters
@@ -179,28 +286,44 @@ class Pipeline(BaseEstimator):
         y : iterable, default=None
             Training targets. Must fulfill label requirements for all steps of
             the pipeline.
+
+        **fit_params : dict of string -> object
+            Parameters passed to the ``fit`` method of each step, where
+            each parameter name is prefixed such that parameter ``p`` for step
+            ``s`` has key ``s__p``.
+
+        Returns
+        -------
+        Xt : array-like, shape = [n_samples, n_transformed_features]
+            Transformed samples
         """
-        Xt, fit_params = self._pre_transform(X, y, **fit_params)
-        if hasattr(self.steps[-1][-1], 'fit_transform'):
-            return self.steps[-1][-1].fit_transform(Xt, y, **fit_params)
+        last_step = self._final_estimator
+        Xt, fit_params = self._fit(X, y, **fit_params)
+        if hasattr(last_step, 'fit_transform'):
+            return last_step.fit_transform(Xt, y, **fit_params)
+        elif last_step is None:
+            return Xt
         else:
-            return self.steps[-1][-1].fit(Xt, y, **fit_params).transform(Xt)
+            return last_step.fit(Xt, y, **fit_params).transform(Xt)
 
     @if_delegate_has_method(delegate='_final_estimator')
     def predict(self, X):
-        """Applies transforms to the data, and the predict method of the
-        final estimator. Valid only if the final estimator implements
-        predict.
+        """Apply transforms to the data, and predict with the final estimator
 
         Parameters
         ----------
         X : iterable
-            Data to predict on. Must fulfill input requirements of first step of
-            the pipeline.
+            Data to predict on. Must fulfill input requirements of first step
+            of the pipeline.
+
+        Returns
+        -------
+        y_pred : array-like
         """
         Xt = X
         for name, transform in self.steps[:-1]:
-            Xt = transform.transform(Xt)
+            if transform is not None:
+                Xt = transform.transform(Xt)
         return self.steps[-1][-1].predict(Xt)
 
     @if_delegate_has_method(delegate='_final_estimator')
@@ -216,122 +339,169 @@ class Pipeline(BaseEstimator):
         X : iterable
             Training data. Must fulfill input requirements of first step of
             the pipeline.
+
         y : iterable, default=None
             Training targets. Must fulfill label requirements for all steps
             of the pipeline.
+
+        **fit_params : dict of string -> object
+            Parameters passed to the ``fit`` method of each step, where
+            each parameter name is prefixed such that parameter ``p`` for step
+            ``s`` has key ``s__p``.
+
+        Returns
+        -------
+        y_pred : array-like
         """
-        Xt, fit_params = self._pre_transform(X, y, **fit_params)
+        Xt, fit_params = self._fit(X, y, **fit_params)
         return self.steps[-1][-1].fit_predict(Xt, y, **fit_params)
 
     @if_delegate_has_method(delegate='_final_estimator')
     def predict_proba(self, X):
-        """Applies transforms to the data, and the predict_proba method of the
-        final estimator. Valid only if the final estimator implements
-        predict_proba.
+        """Apply transforms, and predict_proba of the final estimator
 
         Parameters
         ----------
         X : iterable
-            Data to predict on. Must fulfill input requirements of first step of
-            the pipeline.
+            Data to predict on. Must fulfill input requirements of first step
+            of the pipeline.
+
+        Returns
+        -------
+        y_proba : array-like, shape = [n_samples, n_classes]
         """
         Xt = X
         for name, transform in self.steps[:-1]:
-            Xt = transform.transform(Xt)
+            if transform is not None:
+                Xt = transform.transform(Xt)
         return self.steps[-1][-1].predict_proba(Xt)
 
     @if_delegate_has_method(delegate='_final_estimator')
     def decision_function(self, X):
-        """Applies transforms to the data, and the decision_function method of
-        the final estimator. Valid only if the final estimator implements
-        decision_function.
+        """Apply transforms, and decision_function of the final estimator
 
         Parameters
         ----------
         X : iterable
-            Data to predict on. Must fulfill input requirements of first step of
-            the pipeline.
+            Data to predict on. Must fulfill input requirements of first step
+            of the pipeline.
+
+        Returns
+        -------
+        y_score : array-like, shape = [n_samples, n_classes]
         """
         Xt = X
         for name, transform in self.steps[:-1]:
-            Xt = transform.transform(Xt)
+            if transform is not None:
+                Xt = transform.transform(Xt)
         return self.steps[-1][-1].decision_function(Xt)
 
     @if_delegate_has_method(delegate='_final_estimator')
     def predict_log_proba(self, X):
-        """Applies transforms to the data, and the predict_log_proba method of
-        the final estimator. Valid only if the final estimator implements
-        predict_log_proba.
+        """Apply transforms, and predict_log_proba of the final estimator
 
         Parameters
         ----------
         X : iterable
-            Data to predict on. Must fulfill input requirements of first step of
-            the pipeline.
+            Data to predict on. Must fulfill input requirements of first step
+            of the pipeline.
+
+        Returns
+        -------
+        y_score : array-like, shape = [n_samples, n_classes]
         """
         Xt = X
         for name, transform in self.steps[:-1]:
-            Xt = transform.transform(Xt)
+            if transform is not None:
+                Xt = transform.transform(Xt)
         return self.steps[-1][-1].predict_log_proba(Xt)
 
-    @if_delegate_has_method(delegate='_final_estimator')
-    def transform(self, X):
-        """Applies transforms to the data, and the transform method of the
-        final estimator. Valid only if the final estimator implements
-        transform.
+    @property
+    def transform(self):
+        """Apply transforms, and transform with the final estimator
+
+        This also works where final estimator is ``None``: all prior
+        transformations are applied.
 
         Parameters
         ----------
         X : iterable
-            Data to predict on. Must fulfill input requirements of first step of
-            the pipeline.
+            Data to transform. Must fulfill input requirements of first step
+            of the pipeline.
+
+        Returns
+        -------
+        Xt : array-like, shape = [n_samples, n_transformed_features]
         """
+        # _final_estimator is None or has transform, otherwise attribute error
+        if self._final_estimator is not None:
+            self._final_estimator.transform
+        return self._transform
+
+    def _transform(self, X):
         Xt = X
         for name, transform in self.steps:
-            Xt = transform.transform(Xt)
+            if transform is not None:
+                Xt = transform.transform(Xt)
         return Xt
 
-    @if_delegate_has_method(delegate='_final_estimator')
-    def inverse_transform(self, X):
-        """Applies inverse transform to the data.
-        Starts with the last step of the pipeline and applies ``inverse_transform`` in
-        inverse order of the pipeline steps.
-        Valid only if all steps of the pipeline implement inverse_transform.
+    @property
+    def inverse_transform(self):
+        """Apply inverse transformations in reverse order
+
+        All estimators in the pipeline must support ``inverse_transform``.
 
         Parameters
         ----------
-        X : iterable
-            Data to inverse transform. Must fulfill output requirements of the
-            last step of the pipeline.
+        Xt : array-like, shape = [n_samples, n_transformed_features]
+            Data samples, where ``n_samples`` is the number of samples and
+            ``n_features`` is the number of features. Must fulfill
+            input requirements of last step of pipeline's
+            ``inverse_transform`` method.
+
+        Returns
+        -------
+        Xt : array-like, shape = [n_samples, n_features]
         """
-        if X.ndim == 1:
+        # raise AttributeError if necessary for hasattr behaviour
+        for name, transform in self.steps:
+            if transform is not None:
+                transform.inverse_transform
+        return self._inverse_transform
+
+    def _inverse_transform(self, X):
+        if hasattr(X, 'ndim') and X.ndim == 1:
             warn("From version 0.19, a 1d X will not be reshaped in"
                  " pipeline.inverse_transform any more.", FutureWarning)
             X = X[None, :]
         Xt = X
-        for name, step in self.steps[::-1]:
-            Xt = step.inverse_transform(Xt)
+        for name, transform in self.steps[::-1]:
+            if transform is not None:
+                Xt = transform.inverse_transform(Xt)
         return Xt
 
     @if_delegate_has_method(delegate='_final_estimator')
     def score(self, X, y=None):
-        """Applies transforms to the data, and the score method of the
-        final estimator. Valid only if the final estimator implements
-        score.
+        """Apply transforms, and score with the final estimator
 
         Parameters
         ----------
         X : iterable
-            Data to score. Must fulfill input requirements of first step of the
-            pipeline.
+            Data to predict on. Must fulfill input requirements of first step
+            of the pipeline.
 
         y : iterable, default=None
-            Targets used for scoring. Must fulfill label requirements for all steps of
-            the pipeline.
+            Targets used for scoring. Must fulfill label requirements for all
+            steps of the pipeline.
+
+        Returns
+        -------
+        score : float
         """
         Xt = X
         for name, transform in self.steps[:-1]:
-            Xt = transform.transform(Xt)
+            if transform is not None:
+                Xt = transform.transform(Xt)
         return self.steps[-1][-1].score(Xt, y)
 
     @property
@@ -369,17 +539,18 @@ def make_pipeline(*steps):
     """Construct a Pipeline from the given estimators.
 
     This is a shorthand for the Pipeline constructor; it does not require, and
-    does not permit, naming the estimators. Instead, they will be given names
-    automatically based on their types.
+    does not permit, naming the estimators. Instead, their names will be set
+    to the lowercase of their types automatically.
 
     Examples
     --------
     >>> from sklearn.naive_bayes import GaussianNB
     >>> from sklearn.preprocessing import StandardScaler
-    >>> make_pipeline(StandardScaler(), GaussianNB())    # doctest: +NORMALIZE_WHITESPACE
+    >>> make_pipeline(StandardScaler(), GaussianNB(priors=None))
+    ...     # doctest: +NORMALIZE_WHITESPACE
     Pipeline(steps=[('standardscaler',
                      StandardScaler(copy=True, with_mean=True, with_std=True)),
-                    ('gaussiannb', GaussianNB())])
+                    ('gaussiannb', GaussianNB(priors=None))])
 
     Returns
     -------
@@ -392,38 +563,38 @@ def _fit_one_transformer(transformer, X, y):
     return transformer.fit(X, y)
 
 
-def _transform_one(transformer, name, X, transformer_weights):
-    if transformer_weights is not None and name in transformer_weights:
-        # if we have a weight for this transformer, muliply output
-        return transformer.transform(X) * transformer_weights[name]
-    return transformer.transform(X)
+def _transform_one(transformer, name, weight, X):
+    res = transformer.transform(X)
+    # if we have a weight for this transformer, multiply output
+    if weight is None:
+        return res
+    return res * weight
 
 
-def _fit_transform_one(transformer, name, X, y, transformer_weights,
+def _fit_transform_one(transformer, name, weight, X, y,
                        **fit_params):
-    if transformer_weights is not None and name in transformer_weights:
-        # if we have a weight for this transformer, muliply output
-        if hasattr(transformer, 'fit_transform'):
-            X_transformed = transformer.fit_transform(X, y, **fit_params)
-            return X_transformed * transformer_weights[name], transformer
-        else:
-            X_transformed = transformer.fit(X, y, **fit_params).transform(X)
-            return X_transformed * transformer_weights[name], transformer
     if hasattr(transformer, 'fit_transform'):
-        X_transformed = transformer.fit_transform(X, y, **fit_params)
-        return X_transformed, transformer
+        res = transformer.fit_transform(X, y, **fit_params)
     else:
-        X_transformed = transformer.fit(X, y, **fit_params).transform(X)
-        return X_transformed, transformer
+        res = transformer.fit(X, y, **fit_params).transform(X)
+    # if we have a weight for this transformer, multiply output
+    if weight is None:
+        return res, transformer
+    return res * weight, transformer
 
 
-class FeatureUnion(BaseEstimator, TransformerMixin):
+class FeatureUnion(_BasePipeline, TransformerMixin):
     """Concatenates results of multiple transformer objects.
 
     This estimator applies a list of transformer objects in parallel to the
     input data, then concatenates the results. This is useful to combine
     several feature extraction mechanisms into a single transformer.
 
+    Parameters of the transformers may be set using its name and the parameter
+    name separated by a '__'. A transformer may be replaced entirely by
+    setting the parameter with its name to another transformer,
+    or removed by setting to ``None``.
+
     Read more in the :ref:`User Guide <feature_union>`.
 
     Parameters
@@ -441,9 +612,62 @@ class FeatureUnion(BaseEstimator, TransformerMixin):
 
     """
     def __init__(self, transformer_list, n_jobs=1, transformer_weights=None):
-        self.transformer_list = transformer_list
+        self.transformer_list = tosequence(transformer_list)
         self.n_jobs = n_jobs
         self.transformer_weights = transformer_weights
+        self._validate_transformers()
+
+    def get_params(self, deep=True):
+        """Get parameters for this estimator.
+
+        Parameters
+        ----------
+        deep: boolean, optional
+            If True, will return the parameters for this estimator and
+            contained subobjects that are estimators.
+
+        Returns
+        -------
+        params : mapping of string to any
+            Parameter names mapped to their values.
+        """
+        return self._get_params('transformer_list', deep=deep)
+
+    def set_params(self, **kwargs):
+        """Set the parameters of this estimator.
+
+        Valid parameter keys can be listed with ``get_params()``.
+
+        Returns
+        -------
+        self
+        """
+        self._set_params('transformer_list', **kwargs)
+        return self
+
+    def _validate_transformers(self):
+        names, transformers = zip(*self.transformer_list)
+
+        # validate names
+        self._validate_names(names)
+
+        # validate estimators
+        for t in transformers:
+            if t is None:
+                continue
+            if (not (hasattr(t, "fit") or hasattr(t, "fit_transform")) or not
+                    hasattr(t, "transform")):
+                raise TypeError("All estimators should implement fit and "
+                                "transform. '%s' (type %s) doesn't" %
+                                (t, type(t)))
+
+    def _iter(self):
+        """Generate (name, est, weight) tuples excluding None transformers
+        """
+        get_weight = (self.transformer_weights or {}).get
+        return ((name, trans, get_weight(name))
+                for name, trans in self.transformer_list
+                if trans is not None)
 
     def get_feature_names(self):
         """Get feature names from all transformers.
@@ -454,10 +678,11 @@ class FeatureUnion(BaseEstimator, TransformerMixin):
             Names of the features produced by transform.
         """
         feature_names = []
-        for name, trans in self.transformer_list:
+        for name, trans, weight in self._iter():
             if not hasattr(trans, 'get_feature_names'):
-                raise AttributeError("Transformer %s does not provide"
-                                     " get_feature_names." % str(name))
+                raise AttributeError("Transformer %s (type %s) does not "
+                                     "provide get_feature_names."
+                                     % (str(name), type(trans).__name__))
             feature_names.extend([name + "__" + f for f in
                                   trans.get_feature_names()])
         return feature_names
@@ -467,35 +692,50 @@ class FeatureUnion(BaseEstimator, TransformerMixin):
 
         Parameters
         ----------
-        X : array-like or sparse matrix, shape (n_samples, n_features)
+        X : iterable or array-like, depending on transformers
             Input data, used to fit transformers.
+
+        y : array-like, shape (n_samples, ...), optional
+            Targets for supervised learning.
+
+        Returns
+        -------
+        self : FeatureUnion
+            This estimator
         """
+        self._validate_transformers()
         transformers = Parallel(n_jobs=self.n_jobs)(
             delayed(_fit_one_transformer)(trans, X, y)
-            for name, trans in self.transformer_list)
+            for _, trans, _ in self._iter())
         self._update_transformer_list(transformers)
         return self
 
     def fit_transform(self, X, y=None, **fit_params):
-        """Fit all transformers using X, transform the data and concatenate
-        results.
+        """Fit all transformers, transform the data and concatenate results.
 
         Parameters
         ----------
-        X : array-like or sparse matrix, shape (n_samples, n_features)
+        X : iterable or array-like, depending on transformers
             Input data to be transformed.
 
+        y : array-like, shape (n_samples, ...), optional
+            Targets for supervised learning.
+
         Returns
         -------
         X_t : array-like or sparse matrix, shape (n_samples, sum_n_components)
             hstack of results of transformers. sum_n_components is the
             sum of n_components (output dimension) over transformers.
         """
+        self._validate_transformers()
         result = Parallel(n_jobs=self.n_jobs)(
-            delayed(_fit_transform_one)(trans, name, X, y,
-                                        self.transformer_weights, **fit_params)
-            for name, trans in self.transformer_list)
+            delayed(_fit_transform_one)(trans, name, weight, X, y,
+                                        **fit_params)
+            for name, trans, weight in self._iter())
 
+        if not result:
+            # All transformers are None
+            return np.zeros((X.shape[0], 0))
         Xs, transformers = zip(*result)
         self._update_transformer_list(transformers)
         if any(sparse.issparse(f) for f in Xs):
@@ -509,7 +749,7 @@ class FeatureUnion(BaseEstimator, TransformerMixin):
 
         Parameters
         ----------
-        X : array-like or sparse matrix, shape (n_samples, n_features)
+        X : iterable or array-like, depending on transformers
             Input data to be transformed.
 
         Returns
@@ -519,29 +759,22 @@ class FeatureUnion(BaseEstimator, TransformerMixin):
             sum of n_components (output dimension) over transformers.
         """
         Xs = Parallel(n_jobs=self.n_jobs)(
-            delayed(_transform_one)(trans, name, X, self.transformer_weights)
-            for name, trans in self.transformer_list)
+            delayed(_transform_one)(trans, name, weight, X)
+            for name, trans, weight in self._iter())
+        if not Xs:
+            # All transformers are None
+            return np.zeros((X.shape[0], 0))
         if any(sparse.issparse(f) for f in Xs):
             Xs = sparse.hstack(Xs).tocsr()
         else:
             Xs = np.hstack(Xs)
         return Xs
 
-    def get_params(self, deep=True):
-        if not deep:
-            return super(FeatureUnion, self).get_params(deep=False)
-        else:
-            out = dict(self.transformer_list)
-            for name, trans in self.transformer_list:
-                for key, value in iteritems(trans.get_params(deep=True)):
-                    out['%s__%s' % (name, key)] = value
-            out.update(super(FeatureUnion, self).get_params(deep=False))
-            return out
-
     def _update_transformer_list(self, transformers):
+        transformers = iter(transformers)
         self.transformer_list[:] = [
-            (name, new)
-            for ((name, old), new) in zip(self.transformer_list, transformers)
+            (name, None if old is None else next(transformers))
+            for name, old in self.transformer_list
         ]
 
 
@@ -559,13 +792,16 @@ def make_union(*transformers):
     >>> from sklearn.decomposition import PCA, TruncatedSVD
     >>> make_union(PCA(), TruncatedSVD())    # doctest: +NORMALIZE_WHITESPACE
     FeatureUnion(n_jobs=1,
-                 transformer_list=[('pca', PCA(copy=True, n_components=None,
-                                               whiten=False)),
-                                   ('truncatedsvd',
-                                    TruncatedSVD(algorithm='randomized',
-                                                 n_components=2, n_iter=5,
-                                                 random_state=None, tol=0.0))],
-                 transformer_weights=None)
+           transformer_list=[('pca',
+                              PCA(copy=True, iterated_power='auto',
+                                  n_components=None, random_state=None,
+                                  svd_solver='auto', tol=0.0, whiten=False)),
+                             ('truncatedsvd',
+                              TruncatedSVD(algorithm='randomized',
+                              n_components=2, n_iter=5,
+                              random_state=None, tol=0.0))],
+           transformer_weights=None)
+
 
     Returns
     -------
